<?php
/*
 * Copyright (C) 2007-2014, M2X BV
 *
 * Authors: Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

if (!defined('W2P_BASE_DIR')) {
    die('You should not access this file directly.');
}

// Create new timesheet or edit an existing one.
$timesheet_id = (int)w2PgetParam($_GET, "timesheet_id", 0);

// Timesheet object
$timesheet = new CTimesheet();
$timesheet->timesheet_id = $timesheet_id;

$obj = $timesheet;
$canEdit = $obj->canEdit();
$canDelete = $obj->canDelete();

// check permissions for this record
if (!$canEdit) {
    $AppUI->redirect(ACCESS_DENIED);
}

$obj = $AppUI->restoreObject();
if ($obj) {
    $timesheet = $obj;
    $timesheet_id = $timesheet->timesheet_id;
}
else {
    $timesheet->load(null, $timesheet_id);
}

// load the record data in case of that this script is used to edit the timesheet(timesheet_id(
if (!$timesheet && $timesheet_id > 0) {
    $AppUI->setMsg('Timesheet');
    $AppUI->setMsg('invalidID', UI_MSG_ERROR, true);
    $AppUI->redirect(); // go back to the calling location
}

// Read in all configuration data from timesheet_config
print "configuration options loading";
$config_options = $obj->get_config();
if (!$config_options) {
    $AppUI->setMsg('no configuration options found', UI_MSG_ERROR);
    $AppUI->redirect();
}
print "configuration options loaded";

// setup the title block
// Fill the title block either with 'Edit' or with 'New' depending on 
// if timesheet_id has been transmitted via GET or is empty
$ttl = ( $timesheet_id > 0 ) ? "Edit Timesheet" : "New Timesheet";
$titleBlock = new w2p_Theme_TitleBlock( $ttl, 'timesheet.png', $m, '$m' . '.' . '$a' );
$titleBlock->addCrumb( "?m=timesheet", "view all timesheets" );
if ($canDelete && ($timesheet_id > 0) ) {
    $titleBlock->addCrumbDelete( 'delete timesheet', $canDelete, $msg );
}
$titleBlock->show();

// some javaScript code to submit the form and set the delete object flag for the form processing
?>
<script language="javascript">

    function delIt(id) {
        if (confirm( "<?php echo $AppUI->_('Really delete this timesheet ?');?>" )) {
            var f = document.editFrm;
            f.del.value='1';
            f.submit();
        }
    }

    function changeIt() {
        var f=document.editFrm;
        f.stat.value='1';
        f.submit();
    }

    function periodChange() {
       var f=document.editFrm;
        f.workperiod.value='1';
        f.submit();
    }

    function workedHoursChange(val) {
        var f=document.editFrm;
        f.timesheet_worked.value=val;
    }

</script>
<?php
// use the css-style 'std' of the UI style theme to format the table
// create a form providing to add/edit a timesheet

//Pull all users
$q = new DBQuery();
$q->addQuery('user_id, contact_first_name, contact_last_name');
$q->addTable('users');
$q->addTable('contacts');
$q->addWhere('user_contact = contact_id');
$q->addOrder('contact_last_name, contact_first_name');
$q->exec();
$users = array();
while ( $row = $q->fetchRow()) {
  $users[$row['user_id']] = $row['contact_last_name'] . ', ' . $row['contact_first_name'];
}
// Get current submitter name
if ("$obj->timesheet_creator" == "") {
    $submitter = $users[$AppUI->user_id];
    $obj->timesheet_creator = $AppUI->user_id;
} else {
    $submitter = $users[$obj->timesheet_creator];
}
$q->Clear();

// Get the current timesheet period
$period = new CDate();
if (intval($obj->timesheet_period)) {
    $period->setDate($obj->timesheet_period,DATE_FORMAT_UNIXTIME);
} else {
    // Set the correct year
    if ( $config_options['book_year']['value'] != '' ) {
        $period->setYear( intval($config_options['book_year']['value']) );
    }
    $obj->timesheet_period = $period->getTime();
}

// Get the current timesheet status
$status = $obj->timesheet_status;

// Get the current timesheet creation date
$creation_date = new CDate();
if (intval($obj->timesheet_date)) {
    $creation_date->setDate($obj->timesheet_date,DATE_FORMAT_UNIXTIME);
} else {
    $obj->timesheet_date = $creation_date->getTime();
}

if (intval($obj->timesheet_worked)) {
    $timesheet_worked = $obj->timesheet_worked;
} else {
    $obj->timesheet_worked = 0;
}

// function TimesheetEditForm()
    echo '<table cellspacing="0" cellpadding="4" border="0" width="100%" class="std">';
    echo '<form name="editFrm" action="./index.php?m=timesheet" method="post">';
    // if set, the value of dosql is automatically executed by the dP core application
    // do_timesheet_aed.php will be the target of this form
    // it will execute all database relevant commands
    echo '<input type="hidden" name="dosql" value="do_timesheet_aed" />';

    // the del variable contains a bool flag deciding whether to run a delete operation on the given object with timesheet_id
    // the value of del will be zero by default (do not delete)
    // or in case of mouse click on the delete icon it will set to '1' by javaScript (delete object with given timesheet_id)
    echo '<input type="hidden" name="del" value="0" />';
    echo '<input type="hidden" name="stat" value="0" />';
    echo '<input type="hidden" name="workperiod" value="0" />';

    // the value of timesheet_id will be the id of the timesheet to edit
    // or in case of addition of a new timesheet it will contain '0' as value
    echo '<input type="hidden" name="timesheet_id" value="'.$timesheet_id.'" />';
    echo '<input type="hidden" name="timesheet_date" value="'.$obj->timesheet_date.'" />';
    echo '<input type="hidden" name="timesheet_creator" value="'.$obj->timesheet_creator.'" />';
    echo '<input type="hidden" name="timesheet_worked" value="'.$obj->timesheet_worked.'" />';

    // please notice that html tags that have no </closing tag> should be closed
    // like you find it here (<tag />) for xhtml compliance

    echo '<tr>';
    echo '<td width="50%" valign="top">';
        echo '<table cellspacing="0" cellpadding="2" border="0">';

        echo '<tr>';
        echo '<td align="right" nowrap="nowarp">' . $AppUI->_('Submitter') . ':&nbsp;</td>';
        echo '<td width="100%">' . $submitter .'</td>';
        echo '<td align="right" nowrap="nowarp">' . $AppUI->_('Period') . ':&nbsp;</td>';
        echo '<td width="100%">';

            if ( ($canEdit) && ($status < 1) && ($timesheet_id == 0) ) {
               $newdate = new CDate();
               $newdate->setDate($obj->timesheet_period,DATE_FORMAT_UNIXTIME);
               $newdate->setMonth(1);
        ?>
               <select name="timesheet_period" size="1" class="text" onChange="periodChange();">
               <option <?php if ($period->getMonth() == 1) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('January'); ?></option> <?php $newdate->setMonth(2); ?>
               <option <?php if ($period->getMonth() == 2) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('February'); ?></option> <?php $newdate->setMonth(3); ?>
               <option <?php if ($period->getMonth() == 3) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('March'); ?></option> <?php $newdate->setMonth(4); ?>
               <option <?php if ($period->getMonth() == 4) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('April'); ?></option> <?php $newdate->setMonth(5); ?>
               <option <?php if ($period->getMonth() == 5) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('May'); ?></option> <?php $newdate->setMonth(6); ?>
               <option <?php if ($period->getMonth() == 6) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('June'); ?></option> <?php $newdate->setMonth(7); ?>
               <option <?php if ($period->getMonth() == 7) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('July'); ?></option> <?php $newdate->setMonth(8); ?>
               <option <?php if ($period->getMonth() == 8) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('August'); ?></option> <?php $newdate->setMonth(9); ?>
               <option <?php if ($period->getMonth() == 9) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('September'); ?></option> <?php $newdate->setMonth(10); ?>
               <option <?php if ($period->getMonth() == 10) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('October'); ?></option> <?php $newdate->setMonth(11); ?>
               <option <?php if ($period->getMonth() == 11) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('November'); ?></option> <?php $newdate->setMonth(12); ?>
               <option <?php if ($period->getMonth() == 12) echo "selected"; ?> value=<?php echo '"'. $newdate->getTime() .'"'; ?>><?php echo dPformSafe('December'); ?></option>
               </select>
        <?php
            } else {
                if ($period->getMonth() == 1) echo dPformSafe('January');
                else if ($period->getMonth() == 2) echo dPformSafe('February');
                else if ($period->getMonth() == 3) echo dPformSafe('March');
                else if ($period->getMonth() == 4) echo dPformSafe('April');
                else if ($period->getMonth() == 5) echo dPformSafe('May');
                else if ($period->getMonth() == 6) echo dPformSafe('June');
                else if ($period->getMonth() == 7) echo dPformSafe('July');
                else if ($period->getMonth() == 8) echo dPformSafe('August');
                else if ($period->getMonth() == 9) echo dPformSafe('September');
                else if ($period->getMonth() == 10) echo dPformSafe('October');
                else if ($period->getMonth() == 11) echo dPformSafe('November');
                else if ($period->getMonth() == 12) echo dPformSafe('December');
            }
        echo '</td>';
        echo '</tr>';

        echo '<tr>';
        echo '<td align="right" nowrap="nowrap">' . $AppUI->_('Creation date') . ':&nbsp;</td>';
        echo '<td width="100%">';
            echo $creation_date->getYear() . "-" . $creation_date->getMonth() . "-" . $creation_date->getDay();
        echo '</td>';
        echo '<td align="right" nowrap="nowarp">' . $AppUI->_('Status') . ':&nbsp;</td>';
        echo '<td width="100%" align="right"">';

            if ( ($canEdit) && (($status == 0) ||
                 ($config_options['approval_by']['value'] == $AppUI->user_id)) ) {
                ?>
                <select name="timesheet_status" size="1" class="text" onChange="changeIt()">
                <option <?php if (intval($status) == '0') { echo "selected"; } ?> value="0"> <?php echo dPformSafe('Open for Editing'); ?> </option>
                <option <?php if (intval($status) == '1') { echo "selected"; } ?> value="1"> <?php echo dPformSafe('Submitted'); ?> </option>
                <?php
                // Only the user with Timesheet Approval rights may set a timesheet to approved status.
                if ( $config_options['approval_by']['value'] == $AppUI->user_id ) {
                ?>
                    <option <?php if ($status == '2') { echo "selected"; } ?> value="2"> <?php echo dPformSafe('Approved'); ?> </option>
                <?php
                }
                echo "</select>";
            } else {
                if ($status == 1)
                    echo dPformSafe('Submitted');
                else if ($status == 2)
                    echo dPformSafe('Approved');
            }
        echo '</td>';
        echo '</tr>';
        echo '</table>';

    echo '</td>';
    echo '</tr>';

    echo '<tr>';
    echo '<td>';

    // If $status is not "Open for Editing" then don't allow to change the status.
    // There is one exception to this rule the 'administrator' can always change the status.
    if ( ($canEdit) && (($status == 0) ||
         ($config_options['approval_by']['value'] == $AppUI->user_id)) ) {
    ?>
        <input class="button" type="button" name="cancel" value="<?php echo $AppUI->_('cancel'); ?>" onClick="javascript:if(confirm('Are you sure you want to cancel.')){location.href = './index.php?m=timesheet';}" />
    <?php
        echo '</td>';
        echo '<td align="right">';
        echo '<input class="button" type="submit" name="btnFuseAction" value="' . $AppUI->_('submit') . '"/>';
    } else
        echo '&nbsp;</td><td align="right">&nbsp;';
    echo '</td>';
    echo '</tr>';
    echo '</form>';
    echo '</table>';

// Get Project Status information.
$project_types = dPgetSysVal("ProjectStatus");

// FIXME: Move this function to the Timesheet class
// Find all tasks in this timesheet period and account them to the project.
function showproject()
{
    global $AppUI, $period, $obj, $project_types;

    $q = new DBQuery();
    $q->addQuery('project_id');
    $q->addQuery('project_name');
    $q->addQuery('project_company');
    $q->addQuery('project_status');
    $q->addTable('projects');
    $q->addOrder('project_name ASC');
    $projects = $q->LoadList();
    $timesheet_worked = 0;
    foreach ($projects as $row) {
        $project_id = $row["project_id"];
        $t = new DBQuery;
        $t->addQuery('task_id');
        $t->addQuery('task_name');
        $t->addQuery('task_owner');
        $t->addQuery('task_start_date');
        $t->addQuery('task_hours_worked');
        $t->addTable('tasks');
        $t->addWhere('task_project = '. $project_id);
        $t->addOrder('task_start_date DESC');
        $tasks = $t->LoadList();
        $t->Clear();

        // show company in details
        $company = "unknown";
        $t->addQuery('company_id');
        $t->addQuery('company_name');
        $t->addTable('companies');
        $t->addWhere('company_id = '. $row["project_company"]);
        $companies = $t->LoadList();
        foreach ($companies as $list) {
            $company = $list['company_name'];
        }
        $t->Clear();

        $amount = 0;
        foreach ($tasks as $item) {
            // Query the task_log table for actual start dates.
            $t->addQuery('task_log_id');
            $t->addQuery('task_log_name');
            $t->addQuery('task_log_creator');
            $t->addQuery('task_log_date');
            $t->addQuery('task_log_hours');
            $t->addTable('task_log');
            $t->addWhere('task_log_task = '. $item["task_id"]);
            // What if this is different then current user?
            $t->addWhere('task_log_creator = ' . $AppUI->user_id);
            $t->addOrder('task_log_date DESC');
            // TODO: select on user and date
            $log = $t->LoadList();
            foreach ($log as $logitem) {
                if (intval($logitem['task_log_date'])) {
                    $taskDate = new CDate($logitem['task_log_date']);
                    if ( ($period->GetMonth() == $taskDate->GetMonth()) &&
                         ($period->GetYear()  == $taskDate->GetYear()) ) {
                        $amount = $amount + $logitem['task_log_hours'];
                    }
                    // else do not account the worked hours to the project for this month
                }
            }
        }

        if( $amount > 0 ) {
            echo '<tr>';
            echo '<td>&nbsp;</td>';
            echo '<td><a href="?m=projects&a=view&project_id=' . $row["project_id"] . '">' . $row["project_name"] . '</a></td>';
            echo '<td>'. $company .'</td>';
            echo '<td>' . $amount . '</td>';
            $timesheet_worked = $timesheet_worked + $amount;
            echo '<td>' . $project_types[$row['project_status']] . '</td>';
            echo '</tr>';
        }
    }
    $obj->timesheet_worked = $timesheet_worked;
    echo '<tr><td>&nbsp;</td><td>&nbsp;</td><td><b>'.dPformSafe('Total').'</b></td><td><b>'.$timesheet_worked.'</b></td><td>&nbsp;</td></tr>';
} // end of function showproject()

// Print the overview table
    echo '<table width="100%" border="0" cellpadding="2" cellspacing="1" class="tbl">';
    echo '<tr>';
        echo '<th nowrap="nowrap">&nbsp;</th>';
        echo '<th nowrap="nowrap">'. $AppUI->_( 'Project' ) .'</th>';
        echo '<th nowrap="nowrap">'. $AppUI->_( 'Customer' ) .'</th>';
        echo '<th nowrap="nowrap">'. $AppUI->_( 'Worked hours' ) .'</th>';
        echo '<th nowrap="nowrap">'. $AppUI->_( 'Project status' ) .'</th>';
    echo '</tr>';

    echo '<form name="projectFrm" action="./index.php?m=timesheet" method="post">';
    echo '<input type="hidden" name="dosql" value="do_timesheet_aed" />';
    echo '<input type="hidden" name="projects" value="0" />';

    echo '<input type="hidden" name="timesheet_id" value="'. $timesheet_id .'" />';
    echo '<input type="hidden" name="timesheet_date" value="'. $obj->timesheet_date .'" />';
    echo '<input type="hidden" name="timesheet_creator" value="'. $obj->timesheet_creator .'" />';
    echo '<input type="hidden" name="timesheet_worked" value="'. $obj->timesheet_worked .'" />';

    showproject();

    echo '<script language="javascript">workedHoursChange('. $obj->timesheet_worked .');</script>';
    echo "</form>";
    echo "</table>";
?>
